@@ -3,7 +3,7 @@
  * hashfunc.c
  *	  Support functions for hash access method.
  *
- * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -26,14 +26,18 @@
 
 #include "postgres.h"
 
-#ifdef _MSC_VER
-#include <float.h>				/* for _isnan */
-#endif
-#include <math.h>
-
 #include "access/hash.h"
 #include "utils/builtins.h"
 
+/*
+ * Datatype-specific hash functions.
+ *
+ * These support both hash indexes and hash joins.
+ *
+ * NOTE: some of these are also used by catcache operations, without
+ * any direct connection to hash indexes.  Also, the common hash_any
+ * routine is also used by dynahash tables.
+ */
 
 /* Note: this is used for both "char" and boolean datatypes */
 Datum
@@ -109,16 +113,6 @@ hashfloat4(PG_FUNCTION_ARGS)
 	 */
 	key8 = key;
 
-	/*
-	 * Similarly, NaNs can have different bit patterns but they should all
-	 * compare as equal.  For backwards-compatibility reasons we force them to
-	 * have the hash value of a standard float8 NaN.  (You'd think we could
-	 * replace key with a float4 NaN and then widen it; but on some old
-	 * platforms, that way produces a different bit pattern.)
-	 */
-	if (isnan(key8))
-		key8 = get_float8_nan();
-
 	return hash_any((unsigned char *) &key8, sizeof(key8));
 }
 
@@ -135,14 +129,6 @@ hashfloat8(PG_FUNCTION_ARGS)
 	if (key == (float8) 0)
 		PG_RETURN_UINT32(0);
 
-	/*
-	 * Similarly, NaNs can have different bit patterns but they should all
-	 * compare as equal.  For backwards-compatibility reasons we force them to
-	 * have the hash value of a standard NaN.
-	 */
-	if (isnan(key))
-		key = get_float8_nan();
-
 	return hash_any((unsigned char *) &key, sizeof(key));
 }
 
@@ -154,14 +140,6 @@ hashoidvector(PG_FUNCTION_ARGS)
 	return hash_any((unsigned char *) key->values, key->dim1 * sizeof(Oid));
 }
 
-Datum
-hashint2vector(PG_FUNCTION_ARGS)
-{
-	int2vector *key = (int2vector *) PG_GETARG_POINTER(0);
-
-	return hash_any((unsigned char *) key->values, key->dim1 * sizeof(int16));
-}
-
 Datum
 hashname(PG_FUNCTION_ARGS)
 {
@@ -434,7 +412,7 @@ hash_any(register const unsigned char *k, register int keylen)
 				a += k[0];
 				/* case 0: nothing left to add */
 		}
-#endif   /* WORDS_BIGENDIAN */
+#endif							/* WORDS_BIGENDIAN */
 	}
 	else
 	{
@@ -451,7 +429,7 @@ hash_any(register const unsigned char *k, register int keylen)
 			a += (k[0] + ((uint32) k[1] << 8) + ((uint32) k[2] << 16) + ((uint32) k[3] << 24));
 			b += (k[4] + ((uint32) k[5] << 8) + ((uint32) k[6] << 16) + ((uint32) k[7] << 24));
 			c += (k[8] + ((uint32) k[9] << 8) + ((uint32) k[10] << 16) + ((uint32) k[11] << 24));
-#endif   /* WORDS_BIGENDIAN */
+#endif							/* WORDS_BIGENDIAN */
 			mix(a, b, c);
 			k += 12;
 			len -= 12;
@@ -514,7 +492,7 @@ hash_any(register const unsigned char *k, register int keylen)
 				a += k[0];
 				/* case 0: nothing left to add */
 		}
-#endif   /* WORDS_BIGENDIAN */
+#endif							/* WORDS_BIGENDIAN */
 	}
 
 	final(a, b, c);
