@@ -1650,9 +1650,9 @@ GenerateRecoveryConf(PGconn *conn)
 
 	if (replication_slot)
 	{
-		escaped = escape_quotes(replication_slot);
-		appendPQExpBuffer(recoveryconfcontents, "primary_slot_name = '%s'\n", replication_slot);
-		free(escaped);
+		/* unescaped: ReplicationSlotValidateName allows [a-z0-9_] only */
+		appendPQExpBuffer(recoveryconfcontents, "primary_slot_name = '%s'\n",
+						  replication_slot);
 	}
 
 	if (PQExpBufferBroken(recoveryconfcontents) ||
